(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (factory((global.Lkr = global.Lkr || {})));
}(this, (function (exports) { 'use strict';

/**
 * Module for checking variable types.
 *
 * @module type
 */
var type = {};
var types = 'Array Object String Date RegExp Function Boolean Number Error Null Undefined Symbol'.split(' ');

/**
 * Check the type of a value.
 *
 * @param  {String}  type  The primitive type as a capitalized string.
 * @param  {Mixed}   val   The value to check.
 *
 * @return {Boolean}
 */
type.is = function (type, val) {
  return Object.prototype.toString.call(val).slice(8, -1) === type;
};

// Populate helper methods.
types.forEach(function (t) {
  return type['is' + t] = function (v) {
    return type.is(t, v);
  };
});

/**
 * Return the value or execute as a function
 *
 * @module  value
 *
 * @param   {Mixed}  val     A value or a function that returns a value
 * @param   {Mixed}  [args]  One or more arguments
 *
 * @return  {Mixed}
 */
function value(val) {
  for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  return type.isFunction(val) ? val.apply(this, args) : val;
}

/**
 * Round a number to a given number of decimal places.
 *
 * @param   {Number}  val         The number to round
 * @param   {Number}  [places=2]  The number of places to round to
 *
 * @return  {Number}
 */

// export { default as Str } from './Str';

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();







var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

/**
 * Serialize and Unserialize data
 *
 * @author Sean Tymon <tymon148@gmail.com>
 */
var JSONSerializer = function () {
  function JSONSerializer() {
    classCallCheck(this, JSONSerializer);
  }

  createClass(JSONSerializer, null, [{
    key: "serialize",


    /**
     * Serialize the provided data.
     *
     * @param   {Mixed}  data  The data to serialize
     *
     * @return  {String}
     */
    value: function serialize(data) {
      try {
        return JSON.stringify(data);
      } catch (e) {
        return data;
      }
    }

    /**
     * Unserialize the provided data.
     *
     * @param   {String}  payload  The payload to unserialize
     *
     * @return  {Mixed}
     */

  }, {
    key: "unserialize",
    value: function unserialize(payload) {
      try {
        return JSON.parse(payload);
      } catch (e) {
        return payload;
      }
    }
  }]);
  return JSONSerializer;
}();

/**
 * A decorator for a Storage implementation
 *
 * @author Sean Tymon <tymon148@gmail.com>
 */

var Store = function () {

  /**
   * Create a Store instance.
   *
   * @param   {Storage}  driver        The storage driver
   * @param   {Object}   [serializer]  The serializer to use
   *
   * @return  {void}
   */
  function Store(driver, serializer) {
    classCallCheck(this, Store);

    this.driver = driver;
    this.serializer = serializer || JSONSerializer;
  }

  /**
   * Add a key/value pair to storage
   *
   * @param  {String}  key  The key to add
   * @param  {Mixed}   val  The value to add
   *
   * @throws {Error}  If the storage access is disallowed.
   *
   * @return {Store}
   */


  createClass(Store, [{
    key: 'setItem',
    value: function setItem(key, val) {
      try {
        this.driver.setItem(key, this.serializer.serialize(val));
      } catch (e) {
        if (['QUOTA_EXCEEDED_ERR', 'NS_ERROR_DOM_QUOTA_REACHED', 'QuotaExceededError'].indexOf(e.name) !== -1) {
          throw new Error('The Storage quota has been exceeded');
        } else {
          throw new Error('Could not add item with key "' + key + '"');
        }
      }

      return this;
    }

    /**
     * Retrieve a value from storage by key
     *
     * @param  {String}  key  The key to retreive
     *
     * @return {Mixed}
     */

  }, {
    key: 'getItem',
    value: function getItem(key) {
      return this.serializer.unserialize(this.driver.getItem(key));
    }

    /**
     * Determine whether the key exists in storage.
     *
     * @param   {String}   key  The key to check
     *
     * @return  {Boolean}
     */

  }, {
    key: 'hasItem',
    value: function hasItem(key) {
      return this.driver.hasOwnProperty(key) || !!this.getItem(key);
    }

    /**
     * Remove an item from storage.
     *
     * @param   {String}  key  The key to remove
     *
     * @return  {Boolean}
     */

  }, {
    key: 'removeItem',
    value: function removeItem(key) {
      if (!this.hasItem(key)) return false;

      this.driver.removeItem(key);

      return true;
    }

    /**
     * Remove all items from storage.
     *
     * @return  {Store}
     */

  }, {
    key: 'clear',
    value: function clear() {
      this.driver.clear();

      return this;
    }

    /**
     * Iterate through the items in storage and execute the callback.
     *
     * @param   {Function}  callback            The callback function
     * @param   {Object}    [thisContext=this]  The context of this keyword
     *
     * @return  {void}
     */

  }, {
    key: 'forEach',
    value: function forEach(callback) {
      var thisContext = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this;

      for (var item in this.driver) {
        callback.call(thisContext, this.driver[item], item);
      }
    }

    /**
     * Determine whether the storage driver is supported.
     *
     * @return  {Boolean}
     */

  }, {
    key: 'isSupported',
    value: function isSupported() {
      var l = 'l';
      try {
        this.driver.setItem(l, l);
        this.driver.removeItem(l);

        return true;
      } catch (e) {
        return false;
      }
    }

    /**
     * Get the number of items in storage.
     *
     * @return  {Integer}
     */

  }, {
    key: 'length',
    get: function get$$1() {
      return this.driver.length;
    }
  }]);
  return Store;
}();

/**
 * A Fluent Storage API.
 *
 * @author Sean Tymon <tymon148@gmail.com>
 */

var Lkr = function () {

  /**
   * Create a Lkr instance.
   *
   * @param   {Object}  options               The configuration options
   * @param   {Object}  options.drivers       The storage drivers
   * @param   {String}  options.driver        The default storage driver
   * @param   {String}  options.namespace     The namespace
   * @param   {String}  options.separator     The string that separates the namespace and key
   * @param   {Object}  [options.serializer]  The serializer to use
   *
   * @return  {void}
   */
  function Lkr(options) {
    classCallCheck(this, Lkr);


    /**
     * The configuration options.
     *
     * @type  {Object}
     */
    this.opts = options;

    /**
     * Throw a namespaced Error.
     *
     * @param  {String}  msg  The error message
     *
     * @private
     *
     * @throws {Error}
     *
     * @return {void}
     */
    this._err = function (msg) {
      throw new Error('[lkr] ' + msg);
    };

    /**
     * The Store instance.
     *
     * @private
     *
     * @type {Store}
     */
    this._store = function (driver, self) {
      if (!Object.keys(options.drivers).length || !options.drivers.hasOwnProperty(driver)) {
        self._err('Driver "' + driver + '" not available.');
      }

      var store = new Store(options.drivers[driver], options.serializer);

      if (!store.isSupported()) {
        self._err('Driver "' + driver + '" not supported.');
      }

      return store;
    }(options.driver, this);

    /**
     * Get the fully qualified key.
     *
     * @private
     *
     * @param  {String}  key  The key
     *
     * @return {String}
     */
    this._getKey = function (key) {
      return options.namespace ? '' + options.namespace + options.separator + key : key;
    };
  }

  /**
   * Get the configuration options.
   *
   * @return  {Object}
   */


  createClass(Lkr, [{
    key: 'put',


    /**
     * Add a new item to storage even if it already exists.
     *
     * @param  {String|Function|Object}  key    The key to add
     * @param  {Mixed}                   val    The value to add
     * @param  {Mixed}                   [def]  The default to pass to function if doesn't already exist
     *
     * @throws {Error}  If a key or value is not provided
     *
     * @return {Lkr}
     */
    value: function put(key, val, def) {
      if (type.isUndefined(key)) this._err('You must specify a key.');
      key = value(key);

      if (type.isObject(key)) {
        for (var item in key) {
          var v = value(key[item]);
          this._store.setItem(this._getKey(item), type.isUndefined(v) ? def : v);
        }
      } else {
        if (type.isUndefined(val)) this._err('You must specify a value.');
        this._store.setItem(this._getKey(key), value(val, this.get(key, def)));
      }

      return this;
    }

    /**
     * Add an item to storage if it doesn't already exist.
     *
     * @param  {String|Function|Object}  key      The key to add
     * @param  {Mixed}                   value    The value to add
     * @param  {Mixed}                   [def]    The default to pass to function if doesn't already exist
     *
     * @return {Boolean}  Whether the item was added or not
     */

  }, {
    key: 'add',
    value: function add(key, value$$1, def) {
      if (!this.has(key)) {
        this.put(key, value$$1, def);

        return true;
      }

      return false;
    }

    /**
     * Retrieve the specified item from storage.
     *
     * @param  {String|Array|Function}  key    The key to get
     * @param  {Mixed}                  [def]  The default value if it does not exist
     *
     * @return {Mixed}
     */

  }, {
    key: 'get',
    value: function get$$1(key, def) {
      key = value(key);

      if (type.isArray(key)) {
        var items = {};
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = key[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var k = _step.value;

            items[k] = this.get(k, def);
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }

        return items;
      }

      if (!this.has(key)) return arguments.length === 2 ? def : void 0;

      return this._store.getItem(this._getKey(key));
    }

    /**
     * Determine whether the item exists in storage.
     *
     * @param  {String|Function}  key  The key to remove
     *
     * @return {Boolean}
     */

  }, {
    key: 'has',
    value: function has(key) {
      return this._store.hasItem(value(this._getKey(key)));
    }

    /**
     * Remove specified item(s) from storage.
     *
     * @param  {String|Array|Function}  key  The key or array of keys to remove
     *
     * @return {Lkr}
     */

  }, {
    key: 'forget',
    value: function forget(key) {
      key = value(key);

      if (type.isArray(key)) {
        key.map(this.forget, this);
      } else {
        this._store.removeItem(this._getKey(key));
      }

      return this;
    }

    /**
     * Retrieve the specified item from storage and then remove it
     *
     * @param  {String|Array}  key    The key to pull from storage
     * @param  {Mixed}         [def]  The default value if it does not exist
     *
     * @return {Mixed}
     */

  }, {
    key: 'pull',
    value: function pull(key, def) {
      var value$$1 = this.get(key, def);
      this.forget(key);

      return value$$1;
    }

    /**
     * Get all the items in storage within the currently set namespace/driver.
     *
     * @return  {Object}
     */

  }, {
    key: 'all',
    value: function all() {
      var _this = this;

      var items = {};
      this._store.forEach(function (val, key) {
        if (_this.opts.namespace) {
          var prefix = _this.opts.namespace + _this.opts.separator;
          if (key.indexOf(prefix) === 0) key = key.substring(prefix.length);
        }
        if (_this.has(key)) items[key] = _this.get(key);
      }, this);

      return items;
    }

    /**
     * Iterate through the items within the current namespace/driver and execute the callback.
     *
     * @param   {Function}  callback            The callback function
     * @param   {Object}    [thisContext=this]  The context of this keyword
     *
     * @return  {void}
     */

  }, {
    key: 'each',
    value: function each(callback) {
      var thisContext = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this;

      var items = this.all();
      for (var item in items) {
        callback.call(thisContext, items[item], item);
      }
    }

    /**
     * Get the storage keys as an array.
     *
     * @return {Array}
     */

  }, {
    key: 'keys',
    value: function keys() {
      return Object.keys(this.all());
    }

    /**
     * Remove all items set within the current namespace/driver.
     *
     * @return {Lkr}
     */

  }, {
    key: 'clean',
    value: function clean() {
      return this.forget(this.keys());
    }

    /**
     * Empty the current storage driver completely. Careful now.
     *
     * @return {Lkr}
     */

  }, {
    key: 'empty',
    value: function empty() {
      this._store.clear();

      return this;
    }

    /**
     * Get the total number of items within the current namespace/driver.
     *
     * @return {Integer}
     */

  }, {
    key: 'count',
    value: function count() {
      return this.keys().length;
    }

    /**
     * Set the driver by key.
     *
     * @param   {String}  driver  The driver key
     *
     * @return  {Lkr}
     */

  }, {
    key: 'driver',
    value: function driver(_driver) {
      return Lkr.make(_extends({}, this.opts, { driver: _driver }));
    }

    /**
     * Set the namespace.
     *
     * @param   {String}  namespace  The namespace
     *
     * @return  {Lkr}
     */

  }, {
    key: 'namespace',
    value: function namespace(_namespace) {
      return Lkr.make(_extends({}, this.opts, { namespace: _namespace }));
    }

    /**
     * Create a new instance of Lkr.
     *
     * @param   {Object}  options  The configuration options
     *
     * @return  {Lkr}
     */

  }, {
    key: 'options',
    get: function get$$1() {
      return this.opts;
    }

    /**
     * Get the store instance.
     *
     * @return {Store}
     */

  }, {
    key: 'store',
    get: function get$$1() {
      return this._store;
    }
  }], [{
    key: 'make',
    value: function make(options) {
      return new Lkr(options);
    }
  }]);
  return Lkr;
}();

var Locker = new Lkr({
  drivers: {
    local: window.localStorage,
    session: window.sessionStorage
  },
  driver: 'local',
  namespace: 'lkr',
  separator: '.'
});

exports.Locker = Locker;
exports.Lkr = Lkr;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=lkr.js.map
